@@ -6,6 +6,7 @@ from .shifts import *
 
 from .obs_strategies import *
 from .spectral_trace_list import *
+from .metis_lms_trace_list import *
 from .surface_list import *
 from .ter_curves import *
 from . import ter_curves_utils
@@ -13,5 +14,6 @@ from . import ter_curves_utils
 from .detector_list import *
 from .electronic import *
 from .obs_strategies import *
+from .fits_headers import *
 
 # from . import effects_utils
