@@ -363,7 +363,7 @@ class OpticalTrain:
         iheader = hdulist[1].header
         iheader['EXPTIME'] = from_currsys("!OBS.exptime"), "[s]"
         iheader['DIT'] = from_currsys("!OBS.dit"), "[s]"
-        iheader['NDIT'] = from_currsys("!OBS.ndit"), "[s]"
+        iheader['NDIT'] = from_currsys("!OBS.ndit")
         iheader['BUNIT'] = 'e', 'per EXPTIME'
         iheader['PIXSCALE'] = from_currsys("!INST.pixel_scale"), "[arcsec]"
 
