@@ -144,6 +144,7 @@ class Source(SourceBase):
 
         self.meta = {}
         self.meta.update(kwargs)
+        self._meta_dicts = [self.meta]
 
         self.fields = []
         self.spectra = []
@@ -338,7 +339,8 @@ class Source(SourceBase):
         wave = wcs.all_pix2world(header['CRPIX1'], header['CRPIX2'],
                                  np.arange(data.shape[0]), 0)[-1]
 
-        wave = (wave * u.Unit(wcs.wcs.cunit[-1])).to(u.um)
+        wave = (wave * u.Unit(wcs.wcs.cunit[-1])).to(u.um,
+                                                     equivalencies=u.spectral())
 
         # WCS keywords must be updated because astropy.wcs converts wavelengths to 'm'
         header.update(wcs.to_header())
@@ -505,23 +507,7 @@ class Source(SourceBase):
 
         self.bandpass = bandpass
 
-    def append(self, source_to_add):
-        new_source = source_to_add.make_copy()
-        if isinstance(new_source, Source):
-            for field in new_source.fields:
-                if isinstance(field, Table):
-                    field["ref"] += len(self.spectra)
-                    self.fields += [field]
 
-                elif isinstance(field, (fits.ImageHDU, fits.PrimaryHDU)):
-                    if ("SPEC_REF" in field.header and
-                        isinstance(field.header["SPEC_REF"], int)):
-                        field.header["SPEC_REF"] += len(self.spectra)
-                    self.fields += [field]
-                self.spectra += new_source.spectra
-        else:
-            raise ValueError("Cannot add {} object to Source object"
-                             "".format(type(new_source)))
 
     def plot(self):
         """
@@ -551,6 +537,7 @@ class Source(SourceBase):
     def make_copy(self):
         new_source = Source()
         new_source.meta = deepcopy(self.meta)
+        new_source._meta_dicts = deepcopy(self._meta_dicts)
         new_source.spectra = deepcopy(self.spectra)
         for field in self.fields:
             if isinstance(field, (fits.ImageHDU, fits.PrimaryHDU)) \
@@ -561,6 +548,26 @@ class Source(SourceBase):
 
         return new_source
 
+    def append(self, source_to_add):
+        new_source = source_to_add.make_copy()
+        if isinstance(source_to_add, Source):
+            for field in new_source.fields:
+                if isinstance(field, Table):
+                    field["ref"] += len(self.spectra)
+                    self.fields += [field]
+
+                elif isinstance(field, (fits.ImageHDU, fits.PrimaryHDU)):
+                    if ("SPEC_REF" in field.header and
+                        isinstance(field.header["SPEC_REF"], int)):
+                        field.header["SPEC_REF"] += len(self.spectra)
+                    self.fields += [field]
+                self.spectra += new_source.spectra
+
+                self._meta_dicts += source_to_add._meta_dicts
+        else:
+            raise ValueError("Cannot add {} object to Source object"
+                             "".format(type(new_source)))
+
     def __add__(self, new_source):
         self_copy = self.make_copy()
         self_copy.append(new_source)
